@@ -504,10 +504,6 @@ type UART struct {
 	Interrupt interrupt.Interrupt
 }
 
-var (
-	USB = &USBCDC{Buffer: NewRingBuffer()}
-)
-
 const (
 	sampleRate16X = 16
 	lsbFirst      = 1
@@ -901,6 +897,8 @@ type I2S struct {
 	Bus *sam.I2S_Type
 }
 
+var I2S0 = I2S{Bus: sam.I2S}
+
 // Configure is used to configure the I2S interface. You must call this
 // before you can use the I2S bus.
 func (i2s I2S) Configure(config I2SConfig) {
@@ -1743,6 +1741,10 @@ type USBCDC struct {
 	sent              bool
 }
 
+var (
+	USB = &USBCDC{Buffer: NewRingBuffer()}
+)
+
 const (
 	usbcdcTxSizeMask          uint8 = 0x3F
 	usbcdcTxBankMask          uint8 = ^usbcdcTxSizeMask
@@ -1958,7 +1960,7 @@ func handlePadCalibration() {
 	sam.USB_DEVICE.PADCAL.SetBits(calibTransP << sam.USB_DEVICE_PADCAL_TRANSP_Pos)
 
 	if calibTrim == 0x7 {
-		calibTransN = 3
+		calibTrim = 3
 	}
 	sam.USB_DEVICE.PADCAL.SetBits(calibTrim << sam.USB_DEVICE_PADCAL_TRIM_Pos)
 }
