@@ -58,7 +58,7 @@ func main() {
 	initInternal()
 
 	run()
-	abort()
+	exit(0)
 }
 
 func initSystem() {
@@ -232,6 +232,10 @@ func putchar(c byte) {
 	machine.PutcharUART(machine.UART0, c)
 }
 
+func exit(code int) {
+	abort()
+}
+
 func abort() {
 	println("!!! ABORT !!!")
 
